import { Injectable, inject, signal, computed } from '@angular/core';
import { Observable, catchError, of, tap, map } from 'rxjs';
import { Job, JobStats, JobListResponse } from '../interface/job.interface';
import { JobApiService } from './job-api.service';

@Injectable({ providedIn: 'root' })
export class JobStateService {
  private readonly jobApiService = inject(JobApiService);

  private jobsSignal = signal<Job[]>([]);
  private loadingSignal = signal<boolean>(false);
  private errorSignal = signal<string | null>(null);
  private statsSignal = signal<JobStats | null>(null);

  jobs = this.jobsSignal.asReadonly();
  loading = this.loadingSignal.asReadonly();
  error = this.errorSignal.asReadonly();
  stats = this.statsSignal.asReadonly();

  totalJobs = computed(() => this.stats()?.totalJobs || this.jobs().length);
  companiesCount = computed(() => this.stats()?.companiesCount || new Set(this.jobs().map(job => job.company)).size);

  loadJobs = (): Observable<Job[]> => {
    this.setLoading(true);
    this.clearError();

    return this.jobApiService.getJobs().pipe(
      tap(response => {
        if (response.success) {
          this.setJobs(response.responseData.jobs);
        } else {
          this.setError(response.message || 'Failed to load jobs');
        }
      }),
      map((response: JobListResponse) => {
        return response.success ? response.responseData.jobs : [];
      }),
      catchError((error: any) => {
        this.setError(error.message || 'Failed to load jobs');
        return of([]);
      }),
      tap(() => this.setLoading(false))
    );
  };

  loadJobStats = (): Observable<JobStats | null> => {
    return this.jobApiService.getJobStats().pipe(
      tap(response => {
        if (response && response.success) {
          this.statsSignal.set(response.data);
        }
      }),
      map((response: any) => {
        return response && response.success ? response.data : null;
      }),
      catchError(error => {
        console.error('Failed to load job stats:', error);
        return of(null);
      })
    );
  };

  addJob = (job: Job): void => {
    console.log('Adding new job to state:', job);
    const currentJobs = this.jobsSignal();
    console.log('Current jobs before adding:', currentJobs.length);
    
    const updatedJobs = [job, ...currentJobs];
    this.setJobs(updatedJobs);
    this.updateStats();
    
    console.log('Jobs after adding:', this.jobsSignal().length);
  };

  updateJob = (updatedJob: Job): void => {
    console.log('Updating job in state:', updatedJob);
    const currentJobs = this.jobsSignal();
    console.log('Current jobs before update:', currentJobs.length);
    
    const updatedJobs = currentJobs.map(job => {
      if (job.id === updatedJob.id) {
        console.log('Found job to update:', job.id);
        return updatedJob;
      }
      return job;
    });
    
    this.setJobs(updatedJobs);
    this.updateStats();
    
    console.log('Jobs after update:', this.jobsSignal().length);
  };

  removeJob = (jobId: string): void => {
    const currentJobs = this.jobsSignal();
    this.setJobs(currentJobs.filter(job => job.id !== jobId));
    this.updateStats();
  };

  private setJobs = (jobs: Job[]): void => {
    this.jobsSignal.set(jobs);
  };

  private setLoading = (loading: boolean): void => {
    this.loadingSignal.set(loading);
  };

  private setError = (error: string | null): void => {
    this.errorSignal.set(error);
  };

  private clearError = (): void => {
    this.errorSignal.set(null);
  };

  private updateStats = (): void => {
    const jobs = this.jobsSignal();
    const stats: JobStats = {
      totalJobs: jobs.length,
      companiesCount: new Set(jobs.map(job => job.company)).size
    };
    this.statsSignal.set(stats);
  };
}
